@@ -420,8 +420,8 @@ sub connect
 {
     my $that   = shift( @_ );
     my $param = $that->_connection_params2hash( @_ ) || return;
-    $param->{ 'driver' } = 'mysql';
-    $param->{ 'port' } = 3306 if( !length( $param->{ 'port' } ) );
+    $param->{driver} = 'mysql';
+    $param->{port} = 3306 if( !length( $param->{port} ) );
     return( $that->SUPER::connect( $param ) );
 }
 
